@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
+from fontTools.misc.loggingTools import CapturingLogHandler
 from fontTools.feaLib.error import FeatureLibError
 from fontTools.feaLib.parser import Parser, SymbolTable
-from fontTools.misc.py23 import *
+from io import StringIO
 import warnings
 import fontTools.feaLib.ast as ast
 import os
@@ -62,7 +63,7 @@ class ParserTest(unittest.TestCase):
         glyphMap = {'a': 0, 'b': 1, 'c': 2}
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always")
-            parser = Parser(UnicodeIO(), glyphMap=glyphMap)
+            parser = Parser(StringIO(), glyphMap=glyphMap)
 
             self.assertEqual(len(w), 1)
             self.assertEqual(w[-1].category, UserWarning)
@@ -71,11 +72,11 @@ class ParserTest(unittest.TestCase):
 
             self.assertRaisesRegex(
                 TypeError, "mutually exclusive",
-                Parser, UnicodeIO(), ("a",), glyphMap={"a": 0})
+                Parser, StringIO(), ("a",), glyphMap={"a": 0})
 
             self.assertRaisesRegex(
                 TypeError, "unsupported keyword argument",
-                Parser, UnicodeIO(), foo="bar")
+                Parser, StringIO(), foo="bar")
 
     def test_comments(self):
         doc = self.parse(
@@ -346,6 +347,19 @@ class ParserTest(unittest.TestCase):
         [gc] = self.parse("@range = [A-foo.sc - C-foo.sc];", gn).statements
         self.assertEqual(gc.glyphSet(), ("A-foo.sc", "B-foo.sc", "C-foo.sc"))
 
+    def test_glyphclass_ambiguous_dash_no_glyph_names(self):
+        # If Parser is initialized without a glyphNames parameter (or with empty one)
+        # it cannot distinguish between a glyph name containing an hyphen, or a
+        # range of glyph names; thus it will interpret them as literal glyph names
+        # while also outputting a logging warning to alert user about the ambiguity.
+        # https://github.com/fonttools/fonttools/issues/1768
+        glyphNames = ()
+        with CapturingLogHandler("fontTools.feaLib.parser", level="WARNING") as caplog:
+            [gc] = self.parse("@class = [A-foo.sc B-foo.sc C D];", glyphNames).statements
+        self.assertEqual(gc.glyphSet(), ("A-foo.sc", "B-foo.sc", "C", "D"))
+        self.assertEqual(len(caplog.records), 2)
+        caplog.assertRegex("Ambiguous glyph name that looks like a range:")
+
     def test_glyphclass_glyph_name_should_win_over_range(self):
         # The OpenType Feature File Specification v1.20 makes it clear
         # that if a dashed name could be interpreted either as a glyph name
@@ -704,6 +718,18 @@ class ParserTest(unittest.TestCase):
         self.assertEqual(flag.asFea(),
             "lookupflag RightToLeft MarkAttachmentType @TOP_MARKS;")
 
+    def test_lookupflag_format_A_MarkAttachmentType_glyphClass(self):
+        flag = self.parse_lookupflag_(
+            "lookupflag RightToLeft MarkAttachmentType [acute grave macron];")
+        self.assertIsInstance(flag, ast.LookupFlagStatement)
+        self.assertEqual(flag.value, 1)
+        self.assertIsInstance(flag.markAttachment, ast.GlyphClass)
+        self.assertEqual(flag.markAttachment.glyphSet(),
+                         ("acute", "grave", "macron"))
+        self.assertIsNone(flag.markFilteringSet)
+        self.assertEqual(flag.asFea(),
+            "lookupflag RightToLeft MarkAttachmentType [acute grave macron];")
+
     def test_lookupflag_format_A_UseMarkFilteringSet(self):
         flag = self.parse_lookupflag_(
             "@BOTTOM_MARKS = [cedilla ogonek];"
@@ -717,6 +743,18 @@ class ParserTest(unittest.TestCase):
         self.assertEqual(flag.asFea(),
             "lookupflag IgnoreLigatures UseMarkFilteringSet @BOTTOM_MARKS;")
 
+    def test_lookupflag_format_A_UseMarkFilteringSet_glyphClass(self):
+        flag = self.parse_lookupflag_(
+            "lookupflag UseMarkFilteringSet [cedilla ogonek] IgnoreLigatures;")
+        self.assertIsInstance(flag, ast.LookupFlagStatement)
+        self.assertEqual(flag.value, 4)
+        self.assertIsNone(flag.markAttachment)
+        self.assertIsInstance(flag.markFilteringSet, ast.GlyphClass)
+        self.assertEqual(flag.markFilteringSet.glyphSet(),
+                         ("cedilla", "ogonek"))
+        self.assertEqual(flag.asFea(),
+            "lookupflag IgnoreLigatures UseMarkFilteringSet [cedilla ogonek];")
+
     def test_lookupflag_format_B(self):
         flag = self.parse_lookupflag_("lookupflag 7;")
         self.assertIsInstance(flag, ast.LookupFlagStatement)
@@ -1051,7 +1089,7 @@ class ParserTest(unittest.TestCase):
         self.assertEqual(glyphstr(pos.prefix), "[A a] [B b]")
         self.assertEqual(glyphstr(pos.glyphs), "I [N n] P")
         self.assertEqual(glyphstr(pos.suffix), "[Y y] [Z z]")
-        self.assertEqual(pos.lookups, [lookup1, lookup2, None])
+        self.assertEqual(pos.lookups, [[lookup1], [lookup2], None])
 
     def test_gpos_type_8_lookup_with_values(self):
         self.assertRaisesRegex(
@@ -1494,14 +1532,21 @@ class ParserTest(unittest.TestCase):
     def test_substitute_lookups(self):  # GSUB LookupType 6
         doc = Parser(self.getpath("spec5fi1.fea"), GLYPHNAMES).parse()
         [_, _, _, langsys, ligs, sub, feature] = doc.statements
-        self.assertEqual(feature.statements[0].lookups, [ligs, None, sub])
-        self.assertEqual(feature.statements[1].lookups, [ligs, None, sub])
+        self.assertEqual(feature.statements[0].lookups, [[ligs], None, [sub]])
+        self.assertEqual(feature.statements[1].lookups, [[ligs], None, [sub]])
 
     def test_substitute_missing_by(self):
         self.assertRaisesRegex(
             FeatureLibError,
             'Expected "by", "from" or explicit lookup references',
             self.parse, "feature liga {substitute f f i;} liga;")
+    
+    def test_substitute_invalid_statement(self):
+        self.assertRaisesRegex(
+            FeatureLibError,
+            "Invalid substitution statement",
+            Parser(self.getpath("GSUB_error.fea"), GLYPHNAMES).parse
+        )
 
     def test_subtable(self):
         doc = self.parse("feature test {subtable;} test;")
@@ -1719,8 +1764,14 @@ class ParserTest(unittest.TestCase):
             doc = self.parse("table %s { ;;; } %s;" % (table, table))
             self.assertEqual(doc.statements[0].statements, [])
 
+    def test_ufo_features_parse_include_dir(self):
+        fea_path = self.getpath("include/test.ufo/features.fea")
+        include_dir = os.path.dirname(os.path.dirname(fea_path))
+        doc = Parser(fea_path, includeDir=include_dir).parse()
+        assert len(doc.statements) == 1 and doc.statements[0].text == "# Nothing"
+
     def parse(self, text, glyphNames=GLYPHNAMES, followIncludes=True):
-        featurefile = UnicodeIO(text)
+        featurefile = StringIO(text)
         p = Parser(featurefile, glyphNames, followIncludes=followIncludes)
         return p.parse()
 
