@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
 import os
 import shutil
 import unittest
@@ -10,7 +8,7 @@ from fontTools.ufoLib import plistlib
 from .testSupport import fontInfoVersion2
 
 
-class TestInfoObject(object): pass
+class TestInfoObject: pass
 
 
 class ReadFontInfoVersion2TestCase(unittest.TestCase):
